import java.awt.event.*;
import java.util.*;

/**
 * Lab #5
 * CS 2334, Section SECTION NUMBER GOES HERE
 * DATE GOES HERE
 * <p>
 * Each object of this class represents a GUI enhanced model of a list of candidates.
 * The model of an MVC GUI.
 * </p>
 * @version 1.0
 *
 */
public class CandidatesModel extends Candidates {

	/**
	 * List to keep track of who is registered to listen for events from the CandiatesModel.
	 */
	private ArrayList<ActionListener> actionListenerList;
	
	/**
	 * @param year the year of the election
	 */
	public CandidatesModel(String year) {
		super(year);
	}

	/**
	 * @param year the year of the election
	 * @param candidates the collection of candidates
	 */
	public CandidatesModel(String year, LinkedHashMap<String, Candidate> candidates) {
		super(year, candidates);
	}
	
	/**
	 * @param candidate
	 */
	public void addToCandidates(Candidate candidate) {
		super.addToCandidates(candidate);
		processEvent(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "add candidate"));
	}
	
	
	public void clearCandidates() {
		super.clearCandidates();
		processEvent(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "clear candidate list"));
	}
	
	/**
	 * Register an action event listener.
	 */
	public synchronized void addActionListener(ActionListener l) {
		if (actionListenerList == null)
			actionListenerList = new ArrayList<ActionListener>();
		actionListenerList.add(l);
	}
	
	/**
	 * Remove an action event listener.
	 */
	public synchronized void removeActionListener(ActionListener l) {
		if (actionListenerList != null && actionListenerList.contains(l))
			actionListenerList.remove(l);
	}
	
	/**
	 * Fire event.
	 */
	private void processEvent(ActionEvent e) {
		ArrayList<ActionListener> list;
		
		synchronized (this) {
			if (actionListenerList == null) return;
			list = (ArrayList<ActionListener>)actionListenerList.clone();
		}
		
		for (int i = 0; i < list.size(); i++) {
			ActionListener listener = list.get(i);
			listener.actionPerformed(e);
		}
	}
}